@@ -2,121 +2,95 @@
 # We import the names "as" themselves here (and here only) to tell linting tools
 # that they are explicitly being exported here (and not just unused).
 
-from static_frame.core.util import GetItemKeyType as GetItemKeyType
-from static_frame.core.util import GetItemKeyTypeCompound as GetItemKeyTypeCompound
-from static_frame.core.util import CallableOrMapping as CallableOrMapping
-from static_frame.core.util import KeyOrKeys as KeyOrKeys
-from static_frame.core.util import PathSpecifierOrFileLike as PathSpecifierOrFileLike
-from static_frame.core.util import DtypeSpecifier as DtypeSpecifier
-from static_frame.core.util import IndexSpecifier as IndexSpecifier
-from static_frame.core.util import IndexInitializer as IndexInitializer
-
-from static_frame.core.util import SeriesInitializer as SeriesInitializer
-from static_frame.core.util import FrameInitializer as FrameInitializer
-from static_frame.core.util import mloc as mloc
-
-from static_frame.core.node_selector import InterfaceGetItem as InterfaceGetItem
-
-from static_frame.core.node_iter import IterNodeApplyType as IterNodeApplyType
-from static_frame.core.node_iter import IterNodeType as IterNodeType
-from static_frame.core.node_iter import IterNodeDelegate as IterNodeDelegate
-# from static_frame.core.node_iter import IterNode as IterNode
-
-from static_frame.core.display import DisplayConfig as DisplayConfig
-from static_frame.core.display import DisplayConfigs as DisplayConfigs
-from static_frame.core.display import DisplayActive as DisplayActive
-from static_frame.core.display import Display as Display
-from static_frame.core.display import DisplayFormats as DisplayFormats
-
-from static_frame.core.type_blocks import TypeBlocks as TypeBlocks
-from static_frame.core.index import Index as Index
-from static_frame.core.index import IndexGO as IndexGO
-
-from static_frame.core.index_datetime import IndexYear as IndexYear
-from static_frame.core.index_datetime import IndexYearGO as IndexYearGO
-
-from static_frame.core.index_datetime import IndexYearMonth as IndexYearMonth
-from static_frame.core.index_datetime import IndexYearMonthGO as IndexYearMonthGO
-
-from static_frame.core.index_datetime import IndexDate as IndexDate
-from static_frame.core.index_datetime import IndexDateGO as IndexDateGO
-
-from static_frame.core.index_datetime import IndexMinute as IndexMinute
-from static_frame.core.index_datetime import IndexMinuteGO as IndexMinuteGO
-
-from static_frame.core.index_datetime import IndexHour as IndexHour
-from static_frame.core.index_datetime import IndexHourGO as IndexHourGO
-
-from static_frame.core.index_datetime import IndexSecond as IndexSecond
-from static_frame.core.index_datetime import IndexSecondGO as IndexSecondGO
-
-from static_frame.core.index_datetime import IndexMillisecond as IndexMillisecond
-from static_frame.core.index_datetime import IndexMillisecondGO as IndexMillisecondGO
-
-from static_frame.core.index_datetime import IndexMicrosecond as IndexMicrosecond
-from static_frame.core.index_datetime import IndexMicrosecondGO as IndexMicrosecondGO
-
-from static_frame.core.index_datetime import IndexNanosecond as IndexNanosecond
-from static_frame.core.index_datetime import IndexNanosecondGO as IndexNanosecondGO
-
-from static_frame.core.index import ILoc as ILoc
-from static_frame.core.hloc import HLoc as HLoc
-
-from static_frame.core.index_level import IndexLevel as IndexLevel
-from static_frame.core.index_level import IndexLevelGO as IndexLevelGO
-from static_frame.core.index_hierarchy import IndexHierarchy as IndexHierarchy
-from static_frame.core.index_hierarchy import IndexHierarchyGO as IndexHierarchyGO
-
-from static_frame.core.index_auto import IndexAutoFactory as IndexAutoFactory
-from static_frame.core.index_auto import IndexAutoInitializer as IndexAutoInitializer
-from static_frame.core.index_auto import IndexAutoFactoryType
-
-from static_frame.core.series import Series as Series
-from static_frame.core.series import SeriesAssign as SeriesAssign
-
-from static_frame.core.frame import Frame as Frame
-from static_frame.core.frame import FrameGO as FrameGO
-from static_frame.core.frame import FrameAssign as FrameAssign
-
+from static_frame.core.batch import Batch as Batch
 from static_frame.core.bus import Bus as Bus
-from static_frame.core.store_filter import StoreFilter as StoreFilter
-from static_frame.core.store import StoreConfigMap as StoreConfigMap
-from static_frame.core.store import StoreConfig as StoreConfig
-
+from static_frame.core.display import Display as Display
+from static_frame.core.display import DisplayActive as DisplayActive
+from static_frame.core.interface_meta import InterfaceMeta as InterfaceMeta
+from static_frame.core.display_config import DisplayConfig as DisplayConfig
+from static_frame.core.display_config import DisplayConfigs as DisplayConfigs
+from static_frame.core.display_config import DisplayFormats as DisplayFormats
+from static_frame.core.exception import AxisInvalid
 from static_frame.core.exception import ErrorInit
-from static_frame.core.exception import ErrorInitTypeBlocks
-from static_frame.core.exception import ErrorInitSeries
+from static_frame.core.exception import ErrorInitBus
 from static_frame.core.exception import ErrorInitFrame
 from static_frame.core.exception import ErrorInitIndex
 from static_frame.core.exception import ErrorInitIndexLevel
-from static_frame.core.exception import ErrorInitBus
+from static_frame.core.exception import ErrorInitSeries
 from static_frame.core.exception import ErrorInitStore
 from static_frame.core.exception import ErrorInitStoreConfig
+from static_frame.core.exception import ErrorInitTypeBlocks
 from static_frame.core.exception import LocEmpty
 from static_frame.core.exception import LocInvalid
-from static_frame.core.exception import AxisInvalid
 from static_frame.core.exception import StoreFileMutation
-
-from static_frame.core.node_iter import IterNodeNoArg
+from static_frame.core.frame import Frame as Frame
+from static_frame.core.frame import FrameAssign as FrameAssign
+from static_frame.core.frame import FrameGO as FrameGO
+from static_frame.core.hloc import HLoc as HLoc
+from static_frame.core.index import ILoc as ILoc
+from static_frame.core.index import Index as Index
+from static_frame.core.index import IndexGO as IndexGO
+from static_frame.core.index_auto import IndexAutoFactory as IndexAutoFactory
+from static_frame.core.index_auto import IndexAutoFactoryType
+from static_frame.core.index_auto import IndexAutoInitializer as IndexAutoInitializer
+from static_frame.core.index_datetime import IndexDate as IndexDate
+from static_frame.core.index_datetime import IndexDateGO as IndexDateGO
+from static_frame.core.index_datetime import IndexHour as IndexHour
+from static_frame.core.index_datetime import IndexHourGO as IndexHourGO
+from static_frame.core.index_datetime import IndexMicrosecond as IndexMicrosecond
+from static_frame.core.index_datetime import IndexMicrosecondGO as IndexMicrosecondGO
+from static_frame.core.index_datetime import IndexMillisecond as IndexMillisecond
+from static_frame.core.index_datetime import IndexMillisecondGO as IndexMillisecondGO
+from static_frame.core.index_datetime import IndexMinute as IndexMinute
+from static_frame.core.index_datetime import IndexMinuteGO as IndexMinuteGO
+from static_frame.core.index_datetime import IndexNanosecond as IndexNanosecond
+from static_frame.core.index_datetime import IndexNanosecondGO as IndexNanosecondGO
+from static_frame.core.index_datetime import IndexSecond as IndexSecond
+from static_frame.core.index_datetime import IndexSecondGO as IndexSecondGO
+from static_frame.core.index_datetime import IndexYear as IndexYear
+from static_frame.core.index_datetime import IndexYearGO as IndexYearGO
+from static_frame.core.index_datetime import IndexYearMonth as IndexYearMonth
+from static_frame.core.index_datetime import IndexYearMonthGO as IndexYearMonthGO
+from static_frame.core.index_hierarchy import IndexHierarchy as IndexHierarchy
+from static_frame.core.index_hierarchy import IndexHierarchyGO as IndexHierarchyGO
+from static_frame.core.index_level import IndexLevel as IndexLevel
+from static_frame.core.index_level import IndexLevelGO as IndexLevelGO
+from static_frame.core.node_dt import InterfaceDatetime
+from static_frame.core.node_iter import IterNodeApplyType as IterNodeApplyType
 from static_frame.core.node_iter import IterNodeAxis
-from static_frame.core.node_iter import IterNodeGroup
-from static_frame.core.node_iter import IterNodeGroupAxis
+from static_frame.core.node_iter import IterNodeDelegate as IterNodeDelegate
 from static_frame.core.node_iter import IterNodeDepthLevel
 from static_frame.core.node_iter import IterNodeDepthLevelAxis
+from static_frame.core.node_iter import IterNodeGroup
+from static_frame.core.node_iter import IterNodeGroupAxis
+from static_frame.core.node_iter import IterNodeNoArg
+from static_frame.core.node_iter import IterNodeType as IterNodeType
 from static_frame.core.node_iter import IterNodeWindow
-
-
-from static_frame.core.node_selector import InterfaceSelectDuo
-from static_frame.core.node_selector import InterfaceSelectTrio
-from static_frame.core.node_selector import InterfaceSelectQuartet
-
-from static_frame.core.node_selector import InterfaceAssignTrio
 from static_frame.core.node_selector import InterfaceAssignQuartet
+from static_frame.core.node_selector import InterfaceAssignTrio
 from static_frame.core.node_selector import InterfaceAsType
-
+from static_frame.core.node_selector import InterfaceGetItem as InterfaceGetItem
+from static_frame.core.node_selector import InterfaceSelectDuo
+from static_frame.core.node_selector import InterfaceSelectQuartet
+from static_frame.core.node_selector import InterfaceSelectTrio
 from static_frame.core.node_str import InterfaceString
-from static_frame.core.node_dt import InterfaceDatetime
+from static_frame.core.series import Series as Series
+from static_frame.core.series import SeriesAssign as SeriesAssign
+from static_frame.core.store import StoreConfig as StoreConfig
+from static_frame.core.store import StoreConfigMap as StoreConfigMap
+from static_frame.core.store_filter import StoreFilter as StoreFilter
+from static_frame.core.type_blocks import TypeBlocks as TypeBlocks
+from static_frame.core.util import CallableOrMapping as CallableOrMapping
+from static_frame.core.util import DtypeSpecifier as DtypeSpecifier
+from static_frame.core.util import FrameInitializer as FrameInitializer
+from static_frame.core.util import GetItemKeyType as GetItemKeyType
+from static_frame.core.util import GetItemKeyTypeCompound as GetItemKeyTypeCompound
+from static_frame.core.util import IndexInitializer as IndexInitializer
+from static_frame.core.util import IndexSpecifier as IndexSpecifier
+from static_frame.core.util import KeyOrKeys as KeyOrKeys
+from static_frame.core.util import mloc as mloc
+from static_frame.core.util import PathSpecifierOrFileLike as PathSpecifierOrFileLike
+from static_frame.core.util import SeriesInitializer as SeriesInitializer
 
-
-__version__ = '0.6.24' # use -dev for new version in development
+__version__ = '0.6.35' # use -dev for new version in development
 
