@@ -7,30 +7,23 @@ import numpy as np
 from static_frame.core.container import ContainerBase
 from static_frame.core.display import Display
 from static_frame.core.display import DisplayActive
-from static_frame.core.display import DisplayConfig
 from static_frame.core.display import DisplayHeader
+from static_frame.core.display_config import DisplayConfig
 from static_frame.core.doc_str import doc_inject
 from static_frame.core.exception import ErrorInitBus
 from static_frame.core.frame import Frame
-from static_frame.core.hloc import HLoc
 from static_frame.core.node_selector import InterfaceGetItem
 from static_frame.core.series import Series
 from static_frame.core.store import Store
 from static_frame.core.store import StoreConfigMap
 from static_frame.core.store import StoreConfigMapInitializer
-from static_frame.core.store_hdf5 import StoreHDF5
-from static_frame.core.store_sqlite import StoreSQLite
-from static_frame.core.store_xlsx import StoreXLSX
-from static_frame.core.store_zip import StoreZipCSV
-from static_frame.core.store_zip import StoreZipPickle
-from static_frame.core.store_zip import StoreZipTSV
+from static_frame.core.store_client_mixin import StoreClientMixin
 from static_frame.core.util import DTYPE_BOOL
 from static_frame.core.util import DTYPE_FLOAT_DEFAULT
 from static_frame.core.util import DTYPE_OBJECT
 from static_frame.core.util import GetItemKeyType
+from static_frame.core.util import NameType
 from static_frame.core.util import NULL_SLICE
-from static_frame.core.util import PathSpecifier
-
 
 #-------------------------------------------------------------------------------
 class FrameDefferedMeta(type):
@@ -43,7 +36,10 @@ class FrameDeferred(metaclass=FrameDefferedMeta):
     '''
 
 #-------------------------------------------------------------------------------
-class Bus(ContainerBase): # not a ContainerOperand
+class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
+    '''
+    A lazy, randomly-accessible container of :obj:`Frame`.
+    '''
 
     __slots__ = (
         '_loaded',
@@ -64,7 +60,6 @@ class Bus(ContainerBase): # not a ContainerOperand
         '''
         Return an object ``Series`` of ``FrameDeferred`` objects, based on the passed in ``labels``.
         '''
-        # make an object dtype
         return Series.from_element(FrameDeferred, index=labels, dtype=object)
 
     @classmethod
@@ -72,88 +67,28 @@ class Bus(ContainerBase): # not a ContainerOperand
             frames: tp.Iterable[Frame],
             *,
             config: StoreConfigMapInitializer = None,
+            name: NameType = None,
             ) -> 'Bus':
-        '''Return a ``Bus`` from an iterable of ``Frame``; labels will be drawn from :obj:`Frame.name`.
+        '''Return a :obj:`Bus` from an iterable of :obj:`Frame`; labels will be drawn from :obj:`Frame.name`.
         '''
         # could take a StoreConfigMap
         series = Series.from_items(
                     ((f.name, f) for f in frames),
-                    dtype=object
+                    dtype=object,
+                    name=name,
                     )
         return cls(series, config=config)
 
-    #---------------------------------------------------------------------------
-    # constructors by data format
-
     @classmethod
-    def from_zip_tsv(cls,
-            fp: PathSpecifier,
-            config: StoreConfigMapInitializer = None,
-            ) -> 'Bus':
-        # take and store a StoreConfigMap
-        store = StoreZipTSV(fp)
-        return cls(cls._deferred_series(store.labels()),
-                store=store,
-                config=config
-                )
-
-    @classmethod
-    def from_zip_csv(cls,
-            fp: PathSpecifier,
+    def _from_store(cls,
+            store: Store,
             config: StoreConfigMapInitializer = None
             ) -> 'Bus':
-        store = StoreZipCSV(fp)
         return cls(cls._deferred_series(store.labels()),
                 store=store,
                 config=config
                 )
 
-    @classmethod
-    def from_zip_pickle(cls,
-            fp: PathSpecifier,
-            config: StoreConfigMapInitializer = None
-            ) -> 'Bus':
-        store = StoreZipPickle(fp)
-        return cls(cls._deferred_series(store.labels()),
-                store=store,
-                config=config
-                )
-
-    @classmethod
-    def from_xlsx(cls,
-            fp: PathSpecifier,
-            config: StoreConfigMapInitializer = None
-            ) -> 'Bus':
-        # how to pass configuration for multiple sheets?
-        store = StoreXLSX(fp)
-        return cls(cls._deferred_series(store.labels()),
-                store=store,
-                config=config
-                )
-
-    @classmethod
-    def from_sqlite(cls,
-            fp: PathSpecifier,
-            config: StoreConfigMapInitializer = None
-            ) -> 'Bus':
-        store = StoreSQLite(fp)
-        return cls(cls._deferred_series(store.labels()),
-                store=store,
-                config=config
-                )
-
-    @classmethod
-    def from_hdf5(cls,
-            fp: PathSpecifier,
-            config: StoreConfigMapInitializer = None
-            ) -> 'Bus':
-        store = StoreHDF5(fp)
-        return cls(cls._deferred_series(store.labels()),
-                store=store,
-                config=config
-                )
-
-
     #---------------------------------------------------------------------------
     def __init__(self,
             series: Series,
@@ -191,14 +126,10 @@ class Bus(ContainerBase): # not a ContainerOperand
         # providing None will result in default; providing a StoreConfig or StoreConfigMap will return an appropriate map
         self._config = StoreConfigMap.from_initializer(config)
 
-
     #---------------------------------------------------------------------------
     # delegation
 
     def __getattr__(self, name: str) -> tp.Any:
-        # if name == 'interface':
-        #     return getattr(self.__class__, 'interface')
-
         try:
             return getattr(self._series, name)
         except AttributeError:
@@ -270,7 +201,7 @@ class Bus(ContainerBase): # not a ContainerOperand
 
     def _extract_loc(self, key: GetItemKeyType) -> 'Bus':
 
-        iloc_key = self._series._index.loc_to_iloc(key) #type: ignore
+        iloc_key = self._series._index.loc_to_iloc(key)
 
         # NOTE: if we update before slicing, we change the local and the object handed back
         self._update_series_cache_iloc(key=iloc_key)
@@ -278,12 +209,11 @@ class Bus(ContainerBase): # not a ContainerOperand
         values = self._series.values[iloc_key]
 
         if not isinstance(values, np.ndarray): # if we have a single element
-            if isinstance(key, HLoc) and key.has_key_multiple():
-                # must return a Series, even though we do not have an array
-                values = np.array(values)
-                values.flags.writeable = False
-            else:
-                return values #type: ignore
+            # NOTE: only support str labels, not IndexHierarchy
+            # if isinstance(key, HLoc) and key.has_key_multiple():
+            #     values = np.array(values)
+            #     values.flags.writeable = False
+            return values #type: ignore
 
         series = Series(values,
                 index=self._series._index.iloc[iloc_key],
@@ -304,8 +234,6 @@ class Bus(ContainerBase): # not a ContainerOperand
         '''
         return self._extract_loc(key)
 
-
-
     #---------------------------------------------------------------------------
     # interfaces
 
@@ -317,7 +245,6 @@ class Bus(ContainerBase): # not a ContainerOperand
     def iloc(self) -> InterfaceGetItem['Bus']:
         return InterfaceGetItem(self._extract_iloc)
 
-
     # ---------------------------------------------------------------------------
     def __reversed__(self) -> tp.Iterator[tp.Hashable]:
         return reversed(self._series._index) #type: ignore
@@ -325,11 +252,10 @@ class Bus(ContainerBase): # not a ContainerOperand
     def __len__(self) -> int:
         return self._series.__len__()
 
-
     #---------------------------------------------------------------------------
     # dictionary-like interface
 
-    def items(self) -> tp.Iterator[tp.Tuple[tp.Any, tp.Any]]:
+    def items(self) -> tp.Iterator[tp.Tuple[str, tp.Any]]:
         '''Iterator of pairs of index label and value.
         '''
         self._update_series_cache_all()
@@ -340,8 +266,7 @@ class Bus(ContainerBase): # not a ContainerOperand
         '''A 1D array of values.
         '''
         self._update_series_cache_all()
-        return self._seires.values
-
+        return self._series.values
 
     #---------------------------------------------------------------------------
     @doc_inject()
@@ -360,7 +285,6 @@ class Bus(ContainerBase): # not a ContainerOperand
                 config=config)
         return self._series._display(config, display_cls)
 
-
     #---------------------------------------------------------------------------
     # extended disciptors
 
@@ -494,58 +418,3 @@ class Bus(ContainerBase): # not a ContainerOperand
                 return False
 
         return True
-
-
-
-
-
-    #---------------------------------------------------------------------------
-    # exporters
-
-    def to_zip_tsv(self,
-            fp: PathSpecifier,
-            config: StoreConfigMapInitializer = None
-            ) -> None:
-        store = StoreZipTSV(fp)
-        config = config if not None else self._config
-        store.write(self.items(), config=config)
-
-    def to_zip_csv(self,
-            fp: PathSpecifier,
-            config: StoreConfigMapInitializer = None
-            ) -> None:
-        store = StoreZipCSV(fp)
-        config = config if not None else self._config
-        store.write(self.items(), config=config)
-
-    def to_zip_pickle(self,
-            fp: PathSpecifier,
-            config: StoreConfigMapInitializer = None
-            ) -> None:
-        store = StoreZipPickle(fp)
-        # config must be None for pickels, will raise otherwise
-        store.write(self.items(), config=config)
-
-    def to_xlsx(self,
-            fp: PathSpecifier,
-            config: StoreConfigMapInitializer = None
-            ) -> None:
-        store = StoreXLSX(fp)
-        config = config if not None else self._config
-        store.write(self.items())
-
-    def to_sqlite(self,
-            fp: PathSpecifier,
-            config: StoreConfigMapInitializer = None
-            ) -> None:
-        store = StoreSQLite(fp)
-        config = config if not None else self._config
-        store.write(self.items())
-
-    def to_hdf5(self,
-            fp: PathSpecifier,
-            config: StoreConfigMapInitializer = None
-            ) -> None:
-        store = StoreHDF5(fp)
-        config = config if not None else self._config
-        store.write(self.items())
