@@ -5,13 +5,13 @@ This module us for utilty functions that take as input and / or return Container
 from collections import defaultdict
 from itertools import zip_longest
 from functools import partial
+import datetime
+from fractions import Fraction
+import typing as tp
 
 import numpy as np
 from numpy import char as npc
 
-import typing as tp
-
-
 from static_frame.core.index_base import IndexBase
 from static_frame.core.util import AnyCallable
 from static_frame.core.util import Bloc2DKeyType
@@ -35,6 +35,10 @@ from static_frame.core.util import slice_to_ascending_slice
 from static_frame.core.util import STATIC_ATTR
 from static_frame.core.util import UFunc
 from static_frame.core.util import ufunc_set_iter
+from static_frame.core.util import INT_TYPES
+from static_frame.core.util import NameType
+
+from static_frame.core.exception import AxisInvalid
 
 if tp.TYPE_CHECKING:
     import pandas as pd #pylint: disable=W0611 #pragma: no cover
@@ -71,8 +75,13 @@ def get_col_dtype_factory(
         raise RuntimeError('cannot lookup dtypes by name without supplied columns labels')
 
     def get_col_dtype(col_idx: int) -> DtypeSpecifier:
+        nonlocal dtypes
         if is_map:
             return dtypes.get(columns[col_idx], None) #type: ignore
+
+        # NOTE: dtypes might be a generator deferred until this function is called; if so, realize here
+        if not hasattr(dtypes, '__len__'):
+            dtypes = tuple(dtypes) #type: ignore
         return dtypes[col_idx] #type: ignore
 
     return get_col_dtype
@@ -88,6 +97,17 @@ def is_static(value: IndexConstructor) -> bool:
     return getattr(value.__self__, STATIC_ATTR) #type: ignore
 
 
+# def is_dataclass(value: tp.Any) -> bool:
+#     '''Determine if dataclasses are available. Might be installed via a pip package.
+#     '''
+#     # introduced in Python 3.7; remove special handling when min req is 3.7
+#     try:
+#         import dataclasses
+#     except ImportError:
+#         return False
+#     return dataclasses.is_dataclass(value)
+
+
 def pandas_version_under_1() -> bool:
     import pandas
     return not hasattr(pandas, 'NA') # object introduced in 1.0
@@ -574,7 +594,7 @@ def rehierarch_from_type_blocks(*,
     Given labels suitable for a hierarchical index, order them into a hierarchy using the given depth_map.
 
     Args:
-        index_cls: provide a class, form which the constructor will be called.
+        index_cls: provide a class, from which the constructor will be called.
     '''
 
     depth = labels.shape[1] # number of columns
@@ -882,3 +902,126 @@ def index_many_set(
             union=union,
             assume_unique=True)
     return _index_many_to_one(indices, cls_default, array_processor)
+
+
+#-------------------------------------------------------------------------------
+def apex_to_name(
+        rows: tp.Sequence[tp.Sequence[tp.Hashable]],
+        depth_level: tp.Optional[DepthLevelSpecifier],
+        axis: int, # 0 is by row (for index, 1 is by column (for columns)
+        axis_depth: int,
+        ) -> NameType:
+    '''
+    Utility for translating apex values (the upper left corner created be index/columns) in the appropriate name.
+    '''
+    if depth_level is None:
+        return None
+    if axis == 0:
+        if isinstance(depth_level, INT_TYPES):
+            row = rows[depth_level]
+            if axis_depth == 1: # return a single label
+                return row[0] if row[0] != '' else None
+            else:
+                return tuple(row)
+        else: # its a list selection
+            targets = [rows[level] for level in depth_level]
+            # combine into tuples
+            if axis_depth == 1:
+                return next(zip(*targets))
+            else:
+                return tuple(zip(*targets))
+    elif axis == 1:
+        if isinstance(depth_level, INT_TYPES):
+            # depth_level refers to position in inner row
+            row = [r[depth_level] for r in rows]
+            if axis_depth == 1: # return a single label
+                return row[0] if row[0] != '' else None
+            else:
+                return tuple(row)
+        else: # its a list selection
+            targets = (tuple(row[level] for level in depth_level) for row in rows) #type: ignore
+            # combine into tuples
+            if axis_depth == 1:
+                return next(targets) #type: ignore
+            else:
+                return tuple(targets)
+
+    raise AxisInvalid(f'invalid axis: {axis}')
+
+
+
+#-------------------------------------------------------------------------------
+
+class MessagePackElement:
+    '''
+    Handle encoding/decoding of elements found in object arrays not well supported by msgpack. Many of these cases were found through Hypothesis testing.
+    '''
+
+    @staticmethod
+    def encode(
+            a: tp.Any,
+            packb: AnyCallable,
+            ) -> tp.Tuple[str, tp.Any]:
+
+        if isinstance(a, datetime.datetime): #msgpack-numpy has an issue with datetime
+            year = str(a.year).zfill(4) #datetime returns inconsistent year string for <4 digit years on some systems
+            d = year + ' ' + a.strftime('%a %b %d %H:%M:%S:%f')
+            return ('DT', d)
+        elif isinstance(a, datetime.date):
+            year = str(a.year).zfill(4) #datetime returns inconsistent year string for <4 digit years on some systems
+            d = year + ' ' + a.strftime('%a %b %d')
+            return ('D', d)
+        elif isinstance(a, datetime.time):
+            return ('T', a.strftime('%H:%M:%S:%f'))
+        elif isinstance(a, np.ndarray): #recursion not covered by msgpack-numpy
+            return ('A', packb(a)) #recurse packb
+        elif isinstance(a, Fraction): #msgpack-numpy has an issue with fractions
+            return ('F',  str(a))
+        elif isinstance(a, int) and len(str(a)) >=19:
+            #msgpack-python has an overflow issue with large ints
+            return ('I', str(a))
+        return ('', a)
+
+
+    @staticmethod
+    def decode(
+            pair: tp.Tuple[str, tp.Any],
+            unpackb: AnyCallable,
+            ) -> tp.Any:
+        dt = datetime.datetime
+
+        (typ, d) = pair
+        if typ == 'DT': #msgpack-numpy has an issue with datetime
+            return dt.strptime(d, '%Y %a %b %d %H:%M:%S:%f')
+        elif typ == 'D':
+            return dt.strptime(d, '%Y %a %b %d').date()
+        elif typ == 'T':
+            return dt.strptime(d, '%H:%M:%S:%f').time()
+        elif typ == 'F': #msgpack-numpy has an issue with fractions
+            return Fraction(d)
+        elif typ == 'I': #msgpack-python has an issue with very large int values
+            return int(d)
+        elif typ == 'A': #recursion not covered by msgpack-numpy
+            return unpackb(d) #recurse unpackb
+        return d
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
