@@ -4,17 +4,16 @@ Tools for documenting the SF interface.
 import typing as tp
 import inspect
 from itertools import chain
+from collections import namedtuple
 
 import numpy as np
 
-
+from static_frame.core.batch import Batch
 from static_frame.core.bus import Bus
-from static_frame.core.container import _RIGHT_OPERATOR_MAP
-from static_frame.core.container import _UFUNC_BINARY_OPERATORS
-from static_frame.core.container import _UFUNC_UNARY_OPERATORS
 from static_frame.core.container import ContainerBase
 from static_frame.core.container import ContainerOperand
 from static_frame.core.display import Display
+from static_frame.core.display_config import DisplayConfig
 from static_frame.core.frame import Frame
 from static_frame.core.frame import FrameAsType
 from static_frame.core.index_base import IndexBase
@@ -33,11 +32,77 @@ from static_frame.core.node_selector import InterfaceSelectDuo
 from static_frame.core.node_selector import InterfaceSelectTrio
 from static_frame.core.node_selector import TContainer
 from static_frame.core.node_str import InterfaceString
+from static_frame.core.store import StoreConfig
+from static_frame.core.store_filter import StoreFilter
 from static_frame.core.type_blocks import TypeBlocks
 from static_frame.core.util import AnyCallable
 from static_frame.core.util import DT64_S
 
 
+
+#-------------------------------------------------------------------------------
+
+UFUNC_UNARY_OPERATORS = frozenset((
+        '__pos__',
+        '__neg__',
+        '__abs__',
+        '__invert__',
+        ))
+
+UFUNC_BINARY_OPERATORS = frozenset((
+        '__add__',
+        '__sub__',
+        '__mul__',
+        '__matmul__',
+        '__truediv__',
+        '__floordiv__',
+        '__mod__',
+        #'__divmod__', this returns two np.arrays when called on an np array
+        '__pow__',
+        '__lshift__',
+        '__rshift__',
+        '__and__',
+        '__xor__',
+        '__or__',
+        '__lt__',
+        '__le__',
+        '__eq__',
+        '__ne__',
+        '__gt__',
+        '__ge__',
+        ))
+
+RIGHT_OPERATOR_MAP = frozenset((
+        '__radd__',
+        '__rsub__',
+        '__rmul__',
+        '__rmatmul__',
+        '__rtruediv__',
+        '__rfloordiv__',
+        ))
+
+# reference attributes for ufunc interface testing
+UfuncSkipnaAttrs = namedtuple('UfuncSkipnaAttrs', ('ufunc', 'ufunc_skipna'))
+
+UFUNC_AXIS_SKIPNA: tp.Dict[str, UfuncSkipnaAttrs] = {
+        # 'all': UfuncSkipnaAttrs(ufunc_all, ufunc_nanall),
+        # 'any': UfuncSkipnaAttrs(ufunc_any, ufunc_nanany),
+        'sum': UfuncSkipnaAttrs(np.sum, np.nansum),
+        'min': UfuncSkipnaAttrs( np.min, np.nanmin),
+        'max': UfuncSkipnaAttrs(np.max, np.nanmax),
+        'mean': UfuncSkipnaAttrs(np.mean, np.nanmean),
+        'median': UfuncSkipnaAttrs(np.median, np.nanmedian),
+        'std': UfuncSkipnaAttrs(np.std, np.nanstd),
+        'var': UfuncSkipnaAttrs(np.var, np.nanvar),
+        'prod': UfuncSkipnaAttrs(np.prod, np.nanprod),
+        }
+
+# ufuncs that retain the shape and dimensionality
+UFUNC_SHAPE_SKIPNA: tp.Dict[str, UfuncSkipnaAttrs] = {
+        'cumsum': UfuncSkipnaAttrs(np.cumsum, np.nancumsum),
+        'cumprod': UfuncSkipnaAttrs(np.cumprod, np.nancumprod),
+        }
+
 #-------------------------------------------------------------------------------
 # function inspection utilities
 
@@ -132,7 +197,7 @@ def _get_signatures(
 #-------------------------------------------------------------------------------
 class Features:
     '''
-    Core utilities neede by both Interface and InterfaceSummary
+    Core utilities need by both Interface and InterfaceSummary
     '''
 
     DOC_CHARS = 80
@@ -205,20 +270,22 @@ class Features:
 
 #-------------------------------------------------------------------------------
 class InterfaceGroup:
-    Attribute = 'Attribute'
     Constructor = 'Constructor'
+    Exporter = 'Exporter'
+    Attribute = 'Attribute'
+    Method = 'Method'
     DictLike = 'Dictionary-Like'
     Display = 'Display'
-    Exporter = 'Exporter'
+    Assignment = 'Assignment'
+    Selector = 'Selector'
     Iterator = 'Iterator'
-    Method = 'Method'
     OperatorBinary = 'Operator Binary'
     OperatorUnary = 'Operator Unary'
-    Selector = 'Selector'
-    Assignment = 'Assignment'
-    AccessorString = 'Accessor String'
     AccessorDatetime = 'Accessor Datetime'
+    AccessorString = 'Accessor String'
 
+# NOTE: order from definition retained
+INTERFACE_GROUP_ORDER = tuple(v for k, v in vars(InterfaceGroup).items() if not k.startswith('_'))
 
 class InterfaceRecord(tp.NamedTuple):
 
@@ -647,7 +714,7 @@ class InterfaceRecord(tp.NamedTuple):
 
         signature, signature_no_args = _get_signatures(name, obj, max_args=max_args)
 
-        if name in _UFUNC_UNARY_OPERATORS:
+        if name in UFUNC_UNARY_OPERATORS:
             yield InterfaceRecord(cls_name,
                     InterfaceGroup.OperatorUnary,
                     signature,
@@ -655,7 +722,7 @@ class InterfaceRecord(tp.NamedTuple):
                     reference,
                     signature_no_args=signature_no_args
                     )
-        elif name in _UFUNC_BINARY_OPERATORS or name in _RIGHT_OPERATOR_MAP:
+        elif name in UFUNC_BINARY_OPERATORS or name in RIGHT_OPERATOR_MAP:
             yield InterfaceRecord(cls_name,
                     InterfaceGroup.OperatorBinary,
                     signature,
@@ -678,6 +745,14 @@ class InterfaceRecord(tp.NamedTuple):
 class InterfaceSummary(Features):
 
     _CLS_TO_INSTANCE_CACHE: tp.Dict[tp.Type[ContainerBase], ContainerBase] = {}
+    _CLS_INIT_SIMPLE = frozenset((
+                    ContainerOperand,
+                    ContainerBase,
+                    IndexBase,
+                    DisplayConfig,
+                    StoreFilter,
+                    StoreConfig
+                    ))
 
     @classmethod
     def is_public(cls, field: str) -> bool:
@@ -687,7 +762,6 @@ class InterfaceSummary(Features):
             return False
         return True
 
-
     @classmethod
     def get_instance(cls, target: tp.Type[ContainerBase]) -> ContainerBase:
         '''
@@ -699,14 +773,16 @@ class InterfaceSummary(Features):
             elif target is Bus:
                 f = Frame.from_elements((0,), name='frame')
                 instance = target.from_frames((f,)) #type: ignore
+            elif target is Batch:
+                instance = Batch(iter(()))
             elif issubclass(target, IndexHierarchy):
                 instance = target.from_labels(((0,0),))
             elif issubclass(target, (IndexYearMonth, IndexYear, IndexDate)):
                 instance = target(np.array((0,), dtype=DT64_S))
-            elif target in (ContainerOperand, ContainerBase, IndexBase):
-                instance = target()
             elif issubclass(target, Frame):
                 instance = target.from_elements((0,))
+            elif target in cls._CLS_INIT_SIMPLE:
+                instance = target()
             else:
                 instance = target((0,)) #type: ignore
             cls._CLS_TO_INSTANCE_CACHE[target] = instance
@@ -752,9 +828,7 @@ class InterfaceSummary(Features):
             ) -> tp.Iterator[InterfaceRecord]:
 
         for name_attr, obj, obj_cls in cls.name_obj_iter(target):
-            # properties resdie on the class
             doc = ''
-            # reference = '' # reference attribute to use
 
             if isinstance(obj_cls, property):
                 doc = cls.scrub_doc(obj_cls.__doc__)
@@ -784,9 +858,9 @@ class InterfaceSummary(Features):
                 yield from InterfaceRecord.gen_from_display(**kwargs)
             elif name == 'astype':
                 yield from InterfaceRecord.gen_from_astype(**kwargs)
-            elif name.startswith('from_') or name == '__init__':
+            elif callable(obj) and name.startswith('from_') or name == '__init__':
                 yield from InterfaceRecord.gen_from_constructor(**kwargs)
-            elif name.startswith('to_'):
+            elif callable(obj) and name.startswith('to_'):
                 yield from InterfaceRecord.gen_from_exporter(**kwargs)
             elif name.startswith('iter_'):
                 yield from InterfaceRecord.gen_from_iterator(**kwargs)
@@ -833,9 +907,12 @@ class InterfaceSummary(Features):
         '''
         f = Frame.from_records(
                 cls.interrogate(target, max_args=max_args),
+                )
+        # order be group order
+        f = Frame.from_concat(
+                (f.loc[f['group'] == g] for g in INTERFACE_GROUP_ORDER),
                 name=target.__name__
                 )
-        f = f.sort_values(('cls_name', 'group',))
         f = f.set_index('signature', drop=True)
         if minimized:
             return f[['cls_name', 'group', 'doc']] #type: ignore
