@@ -52,7 +52,10 @@ def get_sample_series_objstr(size: int = 10000) -> tp.Tuple[pd.Series, sf.Series
 # frame generators
 
 
-def get_sample_frame_float_string_index(size: int = 10000, columns: int = 100) -> tp.Tuple[pd.DataFrame, sf.Frame, np.ndarray]:
+def get_sample_frame_float_string_index(
+        size: int = 10000,
+        columns: int = 100
+        ) -> tp.Tuple[pd.DataFrame, sf.Frame, np.ndarray]:
     a1 = (np.arange(size * columns)).reshape((size, columns)) * .001
     # insert random nan in very other columns
     for col in range(0, 100, 2):
@@ -372,6 +375,45 @@ class IndexHierarchy4d_from_labels_str(PerfTest):
         ih = sf.IndexHierarchy.from_labels(SampleData.get('label_tuple4_str'))
 
 
+
+
+
+
+
+class IndexHierarchy2d_iter(PerfTest):
+    NUMBER = 20
+
+    @staticmethod
+    def pd() -> None:
+        post = SampleData.get('pds_float_h2d_str_index').index
+        tuple(post)
+
+    @staticmethod
+    def sf() -> None:
+        post = SampleData.get('sfs_float_h2d_str_index').index
+        tuple(post)
+
+
+class IndexHierarchy3d_iter(PerfTest):
+    NUMBER = 10
+
+    @staticmethod
+    def pd() -> None:
+        post = SampleData.get('pds_float_h3d_str_index').index
+        tuple(post)
+
+    @staticmethod
+    def sf() -> None:
+        post = SampleData.get('sfs_float_h3d_str_index').index
+        tuple(post)
+
+
+
+
+
+
+
+
 #-------------------------------------------------------------------------------
 # series tests
 
@@ -664,7 +706,33 @@ class SeriesIntObjStr_apply(PerfTest):
         post = SampleData.get('sfs_objstr_10k').iter_element().apply(str)
 
 
+class SeriesIntObjStr_reindex_identity(PerfTest):
 
+    NUMBER = 200
+
+    @staticmethod
+    def pd() -> None:
+        s1 = SampleData.get('pds_objstr_10k')
+        s2 = s1.reindex(s1.index)
+
+    @staticmethod
+    def sf() -> None:
+        s1 = SampleData.get('sfs_objstr_10k')
+        s2 = s1.reindex(s1.index)
+
+class SeriesIntObjStr_reindex_reverse(PerfTest):
+
+    NUMBER = 200
+
+    @staticmethod
+    def pd() -> None:
+        s1 = SampleData.get('pds_objstr_10k')
+        s2 = s1.reindex(reversed(s1.index))
+
+    @staticmethod
+    def sf() -> None:
+        s1 = SampleData.get('sfs_objstr_10k')
+        s2 = s1.reindex(reversed(s1.index))
 
 
 
@@ -736,6 +804,7 @@ class SeriesFloatH3DString_loc_slice_slice_target(PerfTest):
         assert len(post) == 1000
 
 
+
 class SeriesIntObj_isin(PerfTest):
     '''isin with objects.
     Numpy's performance as the lookup array grows deteriorates at an exponential rate.
@@ -954,7 +1023,7 @@ class FrameFloat_isna(PerfTest):
 
 class FrameFloat_apply_axis0(PerfTest):
 
-    NUMBER = 100
+    NUMBER = 50
 
     @staticmethod
     def pd() -> None:
@@ -1100,6 +1169,8 @@ class FrameMixed_slice_loc_index(PerfTest):
 
 class FrameMixed_slice_loc_columns(PerfTest):
 
+    NUMBER = 100
+
     @staticmethod
     def pd() -> None:
         for i in range(0, 100, 10):
